/*
 * Copyright (c) 2013-2014, Luis Alvarez <lalvarez@dis.ulpgc.es>
 * All rights reserved.
 *
 * This program is provided for research and education only: you can
 * use and/or modify it for these purposes, but you are not allowed
 * to redistribute this work or derivative works in source or
 * executable form. A license must be obtained from the patent right
 * holders for any other use.
 *
 */

 /**
 * @file image_synthesis.c
 * @brief multilayer image synthesis main program
 *
 * @author Luis Alvarez <lalvarez@dis.ulpgc.es>
 */

#define NB_PALETTE 12 /* NUMBER OF DEFAULT COLOUR PALETTE */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "image_synthesis_lib.h"

/*****************************************************************************************/
/*                              MAIN PROGRAME                                            */
/*****************************************************************************************/


int main(int argc, char *argv[])
{
    int m,size;
	char name[200],name1[200],name2[200],palette_name[200];
    time_t timer;
    ami_object *obj; /* OBJECT POINTER */
    int Nobj; /* NUMBER OF OBJECTS */
    int width,height,width2,height2; /* IMAGE DIMENSIONS IN PIXELS */
    unsigned char *red,*green,*blue; /* POINTER TO PIXEL OUTPUT IMAGE */
    ami_image_information image; /* IMAGE INFORMATION */
    ami_layers_information layers; /* LAYER INFORMATION */
    ami_render render; /* RENDER INFORMATION */
    unsigned int graine,graine2; /* SEED TO INITIALIZE RANDOM FUNCTION */
    double s1,s2;
    int i,k;

    // WE CHECK COMMAND LINE
    if(argc<7){
      printf("FUNCTION SYNTAX\n");
      printf("exe_file  silhouette.png width height output.png output.info color_palette.png [input.info]\n");
      printf("  silhouette.png : silhouette mask to draw the image output\n");
      printf("  width : image width \n");
      printf("  height : image height \n");
      printf("  output.png : image output \n");
      printf("  output.info : image output parameter configuration \n");
      printf("  color_palette.png : color palette image. \n");
      printf("  input.info (optional).\n");
      printf("   (If no configuration file is provided, image parameters are chosen randomly)\n");
      exit(-1);
    }

    // WE FIT IMAGE DIMENSIONS
    width=atoi(argv[2]);
    height=atoi(argv[3]);
    image.width_float=4.; // normalized image width
    image.height_float=4.*height/width; // normalized image height

    // DEFINING IMAGE PARAMETERS
    if (argc==7){
      printf("DEFINING IMAGE PARAMETER CONFIGURATION RANDOMLY\n");
      /* SEED RANDOM NUMBER INITIALIZATION */
	  graine = (unsigned int) time(&timer);
	  //printf("graine=%d",graine);
	  srand(graine);
	  rand();
	  strcpy(palette_name,argv[6]);
	  ami_render_definition(&render,palette_name);
      ami_image_definition(&image);
      ami_layer_definition(&layers,image.Nlayers,image);
      graine = (unsigned int) time(&timer);
	  //printf("graine=%d",graine);
	  srand(graine);
	  rand();
    }
    else{
      printf("READING INFORMATION FILE\n");
      strcpy(name,argv[7]);
      ami_read_basic_image_information(name,&graine,&image,&layers,&render);
      strcpy(render.palette,argv[6]);
      if(ami_read_png(render.palette,&(render.red),&(render.green),
                              &(render.blue),&width2,&height2)!=0){
	    printf("We can not read Palette Color Image\n");
	    exit(-1);
      }
      render.Nc=width2*height2;
      if(argc>8){ 
        if(atoi(argv[8])==1){ // Keep parameters change shape positions
          graine=(unsigned int) time(&timer);
        }
        else if(atoi(argv[8])==2){ // Keep shape positions and change color assignments 
          render.seed=(unsigned int) time(&timer);
        }
        if(argc>9){
          if(atoi(argv[9])==1){ // increase number of layers
            image.max_number_objects*=4;
            ami_add_layer(&layers,image);
          }
		  //decrease number of layers
          else if(atoi(argv[9])==2 && layers.Nlayers>1) ami_remove_layer(&layers);
        }
		if(argc>10){
          if(atoi(argv[10])==1){ // change inclusion randomly
		    graine2 = (unsigned int) time(&timer);
	        srand(graine2);
            for(k=0; k<layers.Nlayers; k++) 
			  layers.inclusion[k]=(k==0)?(((double) rand()/RAND_MAX)<0.99?-1:0):(((double) rand()/RAND_MAX)<0.4?-1:0);
          }
          else if(atoi(argv[10])==2 && layers.Nlayers>1){ // activate inclusion in all layers
		    for(k=0; k<layers.Nlayers; k++) layers.inclusion[k]=0;
		  }
		  else if(atoi(argv[10])==3 && layers.Nlayers>1){ // deactivate inclusion in all layers
		    for(k=0; k<layers.Nlayers; k++) layers.inclusion[k]=-1;
		  }
        }
		if(argc>11){
          if(atoi(argv[11])==1){ // change exclusion randomly
		    graine2 = (unsigned int) time(&timer);
	        srand(graine2);
            for(k=0; k<layers.Nlayers; k++) 
			  layers.exclusion[k]=(k==layers.Nlayers-1)?(((double) rand()/RAND_MAX)<0.99?0:-1):(((double) rand()/RAND_MAX)<0.7?0:-1);
          }
          else if(atoi(argv[11])==2 && layers.Nlayers>1){ // activate exclusion in all layers
		    for(k=0; k<layers.Nlayers; k++) layers.exclusion[k]=0;
		  }
		  else if(atoi(argv[11])==3 && layers.Nlayers>1){ // deactivate exclusion in all layers
		    for(k=0; k<layers.Nlayers; k++) layers.exclusion[k]=-1;
		  }
        }
		if(argc>12){
          if(atoi(argv[12])==1){ // change shape type randomly
		  	graine2 = (unsigned int) time(&timer);
	        srand(graine2);
            for(k=0;k<layers.Nlayers;k++){
              i=layers.object_type[k];
              while(i==layers.object_type[k]){
                layers.object_type[k]=-1+((int)(5*((double) rand()/RAND_MAX)));
              }
			}
          }
          else if(atoi(argv[12])==2 && layers.Nlayers>1){ // all rectangles
		    for(k=0; k<layers.Nlayers; k++) layers.object_type[k]=2;
		  }
		  else if(atoi(argv[12])==3 && layers.Nlayers>1){ // all triangles
		    for(k=0; k<layers.Nlayers; k++) layers.object_type[k]=1;
		  }
		  else if(atoi(argv[12])==4 && layers.Nlayers>1){ // all ellipses
		    for(k=0; k<layers.Nlayers; k++) layers.object_type[k]=0;
		  }
		  else if(atoi(argv[12])==4 && layers.Nlayers>1){ // all polygons
		    for(k=0; k<layers.Nlayers; k++) layers.object_type[k]=4;
		  }
		  else if(atoi(argv[12])==5 && layers.Nlayers>1){ // all random
		    for(k=0; k<layers.Nlayers; k++) layers.object_type[k]=-1;
		  }
        }
        if(argc>13){
          if(atoi(argv[13])==1){ //more shapes
            image.max_number_objects*=4;
            layers.Nobj[0]*=1.5;
            for(k=1;k<layers.Nlayers;k++){ layers.Nobj[k]+=2; }
            for(k=0;k<layers.Nlayers;k++){ layers.Ntrials[k]*=1.5; }
          }
          else if(atoi(argv[13])==2){ //less shapes
            image.max_number_objects/=2;
            layers.Nobj[0]/=1.5;
            for(k=1;k<layers.Nlayers;k++){ if(layers.Nobj[k]>1) layers.Nobj[k]-=1; }
          }
        }
        if(argc>14){
          if(atoi(argv[14])==1){ // bigger shapes
            image.max_diameter*=1.5;         
          }
          else if(atoi(argv[14])==2){
            image.max_diameter/=1.5; //smaller shapes  
          }     
        }
		if(argc>15){
          if(atoi(argv[15])==1){ // increase aspect ratio
            for(k=0; k<layers.Nlayers; k++) layers.ellipse_ratio[k]*=0.8;
          }
          else if(atoi(argv[15])==2){ // decrease aspect ratio
		    for(k=0; k<layers.Nlayers; k++){
 			  layers.ellipse_ratio[k]/=0.8;
			  if(layers.ellipse_ratio[k]>1.) layers.ellipse_ratio[k]=1; 
			}
		  }
        }
		if(argc>16){
          if(atoi(argv[16])==1){ // remove vanishing points in all layers
            for(k=0; k<layers.Nlayers; k++){
			  layers.p[k][0][0]=0; layers.p[k][0][1]=0; layers.p[k][0][2]=0;
              layers.p[k][1][0]=0.; layers.p[k][1][1]=0; layers.p[k][1][2]=0;
			}
          }
          else if(atoi(argv[16])==2){ // add/change vanishing points in all layers 
		  	graine2 = (unsigned int) time(&timer);
	        srand(graine2);
		    for(k=0; k<layers.Nlayers; k++){
 			  layers.p[k][0][0]=image.width_float*((double) rand()/RAND_MAX);
	          layers.p[k][0][1]=image.height_float*((double) rand()/RAND_MAX);
	          layers.p[k][0][2]=((int) (2.*((double) rand()/RAND_MAX)));
	          if( ((double) rand()/RAND_MAX) < 0.6 ){
	            if( ((double) rand()/RAND_MAX) < 0.3){
	              layers.p[k][1][0]=image.width_float*((double) rand()/RAND_MAX);
	              layers.p[k][1][1]=image.height_float*((double) rand()/RAND_MAX);
	              layers.p[k][1][2]=((int) (2.*((double) rand()/RAND_MAX)));
                }
	            else{
		          layers.p[k][0][0]=0; layers.p[k][0][1]=1; layers.p[k][0][2]=0;
                  layers.p[k][1][0]=1.; layers.p[k][1][1]=0; layers.p[k][1][2]=0;
			    }
	          }
	          else{
		        layers.p[k][1][0]=layers.p[k][0][0];
		        layers.p[k][1][1]=layers.p[k][0][1];
		        layers.p[k][1][2]=layers.p[k][0][2];
              }  
			}
		  }
        }
		if(argc>17){
          if(atoi(argv[17])==1){ // higher light spot effect
		    graine2 = (unsigned int) time(&timer);
	        srand(graine2);
		    if(render.light_spot<0.1) render.light_spot=0.5+1.5*((double) rand()/RAND_MAX);
            render.light_spot*=1.5;         
          }
          else if(atoi(argv[17])==2){ // lower light spot effect
		    if(render.light_spot<0.3) render.light_spot=0.;
            else render.light_spot/=1.5; 
          }
          else if(atoi(argv[17])==3){ // no light spot effect
		    render.light_spot=0.; 
		  }
        }
		if(argc>18){
          if(atoi(argv[18])==1){ // more transparency
		    if(render.max_transparency<=0.1){
			  graine2 = (unsigned int) time(&timer);
	          srand(graine2);
              render.min_transparency=0.1+0.6*((double) rand()/RAND_MAX);
              render.max_transparency=
                render.min_transparency+((double) rand()/RAND_MAX)*(1.-render.min_transparency); 
            }	
            render.min_transparency+=(1-render.min_transparency)*0.25;
			render.max_transparency+=(1-render.max_transparency)*0.25;
          }
          else if(atoi(argv[18])==2){ // less transparency
            render.min_transparency=(render.min_transparency-0.25)/0.75;
			render.max_transparency=(render.max_transparency-0.25)/0.75;
			if(render.min_transparency<0) render.min_transparency=0.;
			if(render.max_transparency<0) render.max_transparency=0.;
          }
		  else if(atoi(argv[18])==3){ // no transparency
            render.min_transparency=0.;
			render.max_transparency=0.;
          }
        }
		if(argc>19){
          if(atoi(argv[19])==1){ // no tesselation 
		    graine2 = (unsigned int) time(&timer);
	        srand(graine2);
		    if(render.light_spot<0.) render.light_spot=0.5+1.5*((double) rand()/RAND_MAX);
          }
          else if(atoi(argv[19])==2){ // yes tesselation
		    render.light_spot=-1.; 
          }
        }
		if(argc>20){
          if(atoi(argv[20])==1){ // activate Keep father inclusion 
            render.keepFatherInclusion=0;         
          }
          else if(atoi(argv[20])==2){ // deactivate Keep father inclusion 
		    render.keepFatherInclusion=-1;
          }
        }
		
      }
      srand(graine);
      rand();
    }
    
    /* WE TAKE MEMORY FOR LAYER silhouettes. TO SIMPLIFY WE USE THE SAME silhouette IN ALL LAYERS */
    if(strcmp("null.png",argv[1])==0){
      layers.width=NULL;
      layers.height=NULL;
      layers.red=NULL;
      layers.green=NULL;
      layers.blue=NULL;
    }
    else{
      ami_malloc1d(layers.width, int ,layers.Nlayers);
      ami_malloc1d(layers.height, int ,layers.Nlayers);
      ami_malloc1d(layers.red, unsigned char * ,layers.Nlayers);
      ami_malloc1d(layers.green, unsigned char * ,layers.Nlayers);
      ami_malloc1d(layers.blue, unsigned char * ,layers.Nlayers);
      for(m=0;m<layers.Nlayers;m++){
        ami_read_png(argv[1],&(layers.red[m]),&(layers.green[m]),&(layers.blue[m]),&(layers.width[m]),&(layers.height[m]));
      }
      // WE CHECK IF SILHOUETTE SIZE PROPORTION ARE THE SAME OF IMAGE PROPORTION
      s1=(double) height/width;
      s2=(double) layers.height[0]/layers.width[0];
      if(fabs(s1-s2)>0.000001){
        height=width*s2;
        image.height_float=4.*height/width; // normalized image height
      }
    }

    /* WE FIT IMAGE SIZE AND TAKE MEMORY FOR IMAGE AND WE FIT IMAGE BACKGROUND TO WHITE*/
    size=width*height;
    ami_malloc1d(red,unsigned char,size);
    ami_malloc1d(green,unsigned char,size);
    ami_malloc1d(blue,unsigned char,size);
    for(m=0;m<size;m++){
        red[m]=green[m]=blue[m]=255;
    }
    
    /* WE FIX THE IMAGE AND INFO FILE NAMES */
    //ami_date_name(name);
    strcpy(name1,argv[4]);
    strcpy(name2,argv[5]);

    printf("INITIALIZING OBJECT GENERATION\n");
    Nobj=1;
    ami_image_generator(&obj,&Nobj,image,layers,image.max_number_objects);
    printf("Number of objects created %d\n",Nobj);

    printf("INITIALIZING IMAGE RENDERING\n");
    ami_render_image(Nobj,obj,render,image,red,green,blue,width,height);

    printf("WRITING IMAGE AND INFORMATION FILES\n");
    ami_write_basic_image_information(name2, name1, graine,argv,argc,image,layers,render);
    ami_write_png(name1, red, green, blue, width, height);

    /* WE FREE THE MEMORY */
    if(red!=NULL) free(red);
    if(green!=NULL) free(green);
    if(blue!=NULL) free(blue);
    ami_free_render(&render);
    ami_free_layer(&layers);
    ami_free_objects(&obj,Nobj);

    return 0;
}





