/*
 * Author    : Luis Alvarez  <lalvarez@dis.ulpgc.es>
              Jean-Michel Morel <morel@cmla.ens-cachan.fr>
 * Copyright : (C) 2009-2013 IPOL Image Processing On Line http://www.ipol.im/
 * License   : CC Creative Commons "Attribution-NonCommercial-ShareAlike"
              see http://creativecommons.org/licenses/by-nc-sa/3.0/es/deed.en
*/



 /**
 * @file image_synthesis_lib.c
 * @brief multilayer image synthesis library
 *
 * @author Luis Alvarez <lalvarez@dis.ulpgc.es>
 */

#include "image_synthesis_lib.h"

// FUNCTION TO CHECK IF A POINT IS INSIDE A SILHOUETTE
int mask_inclusion_test(float x,float y,unsigned char *red,int width,int height,float width_float,float height_float){
  int i=x*width/width_float;
  int j=y*height/height_float;
  int m=j*width+i;
  //printf("x=%f y=%f, i=%d j=%d \n",x,y,i,j); //system("pause");
  if(m>=(width*height) || m<0) return(-1);
  if(i<0 || i>(width-1) || j<0 || j>(height-1)) return(-1);
  if(red[m]==0) return(-1);
  if( red[m]<(255.*((double) rand()/RAND_MAX))) return(-1);
  return(0);
}


/* FUNCTION TO CONVERT FROM HSV COLOR SPACE TO RGB */
void ami_hsv_to_rgb(float h,float s,float v,
                    unsigned char *r,unsigned char *g,unsigned char *b)
{
      float
        H = h*360./255.,
        S = s/255.,
        V = v/255.,
        R = 0, G = 0, B = 0;
      if (H==0 && S==0) R = G = B = V;
      else {
        H/=60.;
        const int i = (int)floor((int) H)%6;
        const float
          f = (i&1)?(H - i):(1 - H + i),
          m = V*(1 - S),
          n = V*(1 - S*f);
        switch (i) {
        case 6 :
        case 0 : R = V; G = n; B = m; break;
        case 1 : R = n; G = V; B = m; break;
        case 2 : R = m; G = V; B = n; break;
        case 3 : R = m; G = n; B = V; break;
        case 4 : R = n; G = m; B = V; break;
        case 5 : R = V; G = m; B = n; break;
        }
      }
      R*=255; G*=255; B*=255;
      *r = (unsigned char) (R<0?0:(R>255?255:R));
      *g = (unsigned char) (G<0?0:(G>255?255:G));
      *b = (unsigned char) (B<0?0:(B>255?255:B));
}

/* FUNCTION TO CONVERT FROM RGB COLOR SPACE TO HSV */
void ami_rgb_to_hsv(unsigned char red,unsigned char green,unsigned char blue,
                    float *H,float *S,float *V)
{
    float rgb_min,rgb_max;
    rgb_min=rgb_max=red;
    if(rgb_min>green) rgb_min=green;
    if(rgb_min>blue) rgb_min=blue;
    if(rgb_max<green) rgb_max=green;
    if(rgb_max<blue) rgb_max=blue;
    if(rgb_max == 0){
      *H=0;
      *S=0;
      *V=0;  //HSV
      return;
    }
    *V= rgb_max;   //V
    *S= 255.*(rgb_max - rgb_min)/rgb_max;  //S
    if((*S)==0){
      *H=0; //H
      return;
    }
      //H
    if(rgb_max==red){
      *H=0.+43.*((int) green - blue)/(rgb_max - rgb_min);
    }
    else if (rgb_max == green) {
      *H= 85.+ 43.*((int) blue-red)/(rgb_max - rgb_min);
    }
    else{
      *H= 171. + 43.*((int) red - green)/(rgb_max - rgb_min);
    }
}

/* FUNCTION TO DEFINE RANDOMLY IMAGE PARAMETERS */
void ami_image_definition(ami_image_information *image)
{
  image->max_number_objects=3000; //(int) 3000*(0.5+0.5*(double) rand()/RAND_MAX);
  image->max_diameter=image->width_float/(2.+4.*(double) rand()/RAND_MAX);
  //image->color[0]=255; image->color[1]=255; image->color[2]=255;
  image->Nlayers=2+2.5*((double) rand()/RAND_MAX)	;
}

/* FUNCTION TO DEFINE RANDOMLY LAYER INFORMATION */
void ami_layer_definition(ami_layers_information *layers, int Nlayers,ami_image_information image)
{
  int i,j,k,k2;
  layers->Nlayers=Nlayers;
  ami_malloc1d(layers->min_diameter_ratio, float,layers->Nlayers);
  ami_malloc1d(layers->max_diameter_ratio, float,layers->Nlayers);
  ami_malloc1d(layers->object_type, int ,layers->Nlayers);
  ami_malloc1d(layers->color, vertex_3,layers->Nlayers);
  ami_malloc1d(layers->inclusion, int ,layers->Nlayers);
  ami_malloc1d(layers->exclusion, int ,layers->Nlayers);
  ami_malloc1d(layers->Nobj, int ,layers->Nlayers);
  ami_malloc1d(layers->Ntrials, int ,layers->Nlayers);
  ami_malloc1d(layers->p, vertex_3_f,layers->Nlayers);
  ami_malloc1d(layers->r,float,layers->Nlayers);
  ami_malloc1d(layers->ellipse_ratio,float,layers->Nlayers);
  ami_malloc1d(layers->NdiameterReductions,int,layers->Nlayers);
  ami_malloc1d(layers->diameterReduction,float,layers->Nlayers);
  //BBB
  for(i=0; i<layers->Nlayers; i++){
    layers->max_diameter_ratio[i]=( 0.2+0.8*(double) rand()/RAND_MAX );
	layers->min_diameter_ratio[i]=layers->max_diameter_ratio[i]*
	                              ( ((double) rand()/RAND_MAX)*0.5 +0.5);
	layers->Nobj[i]=(i==0)?500*(0.1+0.9*((double) rand()/RAND_MAX)):
	                     20*(0.2+0.8*((double) rand()/RAND_MAX));
	layers->inclusion[i]=(i==0)?(((double) rand()/RAND_MAX)<0.99?-1:0):
	                              (((double) rand()/RAND_MAX)<0.4?-1:0);
	layers->exclusion[i]=(i==layers->Nlayers-1)? (((double) rand()/RAND_MAX)<0.99?0:-1):
	  (((double) rand()/RAND_MAX)<0.7?0:-1);
	layers->NdiameterReductions[i]=80.+40.*((double) rand()/RAND_MAX);
	layers->diameterReduction[i]=(0.3+0.6*((double) rand()/RAND_MAX));
    layers->object_type[i]=-1+((int)(5*((double) rand()/RAND_MAX)));
	layers->ellipse_ratio[i]=(0.3+0.7*((double) rand()/RAND_MAX));
	layers->r[i]=((double) rand()/RAND_MAX)<0.5?0:1.+5.*((double) rand()/RAND_MAX);
    for(j=0;j<3;j++) layers->color[i][j]=-1;
	layers->Ntrials[i]=80+50*((double) rand()/RAND_MAX);
	layers->p[i][0][0]=image.width_float*((double) rand()/RAND_MAX);
	layers->p[i][0][1]=image.height_float*((double) rand()/RAND_MAX);
	layers->p[i][0][2]=((int) (2.*((double) rand()/RAND_MAX)));
	if( ((double) rand()/RAND_MAX) < 0.6 ){
	  if( ((double) rand()/RAND_MAX) < 0.3){
	    layers->p[i][1][0]=image.width_float*((double) rand()/RAND_MAX);
	    layers->p[i][1][1]=image.height_float*((double) rand()/RAND_MAX);
	    layers->p[i][1][2]=((int) (2.*((double) rand()/RAND_MAX)));
      }
	  else{
		if( ((double) rand()/RAND_MAX) < 0.5){
		  layers->p[i][0][0]=0; layers->p[i][0][1]=0; layers->p[i][0][2]=0;
          layers->p[i][1][0]=0.; layers->p[i][1][1]=0; layers->p[i][1][2]=0;
	    }
		else{
		  layers->p[i][0][0]=0; layers->p[i][0][1]=1; layers->p[i][0][2]=0;
          layers->p[i][1][0]=1.; layers->p[i][1][1]=0; layers->p[i][1][2]=0;
	    }
	  }
    }
	else{
		layers->p[i][1][0]=layers->p[i][0][0];
		layers->p[i][1][1]=layers->p[i][0][1];
		layers->p[i][1][2]=layers->p[i][0][2];
    }
  }
  // WE CHECK IF THE TOTAL NUMBER OF OBJECTS IS NOT BIGGER THAN 3000
  k2=k=layers->Nobj[0];
  for(i=1; i<layers->Nlayers; i++){
    k*=layers->Nobj[i];
    k2+=k;
  }
  while(k2<2500){
    k=1; k2=0;
    for(i=0; i<layers->Nlayers; i++){
      layers->Nobj[i]++;
      k*=layers->Nobj[i];
      k2+=k;
    }
  }
  while(k2>3000){
    k=1; k2=0;
    for(i=0; i<layers->Nlayers; i++){
      if(layers->Nobj[i]>1) layers->Nobj[i]--;
      k*=layers->Nobj[i];
      k2+=k;
    }
  }
}


/* FUNCTION TO ADD 1 LAYER */
void ami_add_layer(ami_layers_information *layers,ami_image_information image)
{
  int i,j,Nl;
  layers->Nlayers++;
  Nl=layers->Nlayers;

  layers->min_diameter_ratio=(float *) realloc(layers->min_diameter_ratio, sizeof(float)*layers->Nlayers);
  layers->max_diameter_ratio=(float *) realloc(layers->max_diameter_ratio, sizeof(float)*layers->Nlayers);
  layers->r=(float *) realloc(layers->r,sizeof(float)*layers->Nlayers);
  layers->ellipse_ratio=(float *) realloc(layers->ellipse_ratio,sizeof(float)*layers->Nlayers);
  layers->diameterReduction=(float *) realloc(layers->diameterReduction,sizeof(float)*layers->Nlayers);
  layers->color=(vertex_3 *) realloc(layers->color, sizeof(vertex_3)*layers->Nlayers);
  layers->p=(vertex_3_f *) realloc(layers->p, sizeof(vertex_3_f)*layers->Nlayers);
  layers->object_type=(int *) realloc(layers->object_type, sizeof(int)*layers->Nlayers);
  layers->inclusion=(int *) realloc(layers->inclusion, sizeof(int)*layers->Nlayers);
  layers->exclusion=(int *) realloc(layers->exclusion, sizeof(int)*layers->Nlayers);
  layers->Nobj=(int *) realloc(layers->Nobj, sizeof(int)*layers->Nlayers);
  layers->Ntrials=(int *) realloc(layers->Ntrials, sizeof(int)*layers->Nlayers);
  layers->NdiameterReductions=(int *) realloc(layers->NdiameterReductions,sizeof(int)*layers->Nlayers);

  //BBB
  for(i=layers->Nlayers-1; i<layers->Nlayers; i++){
    layers->max_diameter_ratio[i]=( 0.1+0.4*(double) rand()/RAND_MAX );
	layers->min_diameter_ratio[i]=layers->max_diameter_ratio[i]*
	                              ( ((double) rand()/RAND_MAX)*0.5 +0.5);
	layers->Nobj[i]=2+3*(0.2+0.8*((double) rand()/RAND_MAX));
	layers->inclusion[i]=(((double) rand()/RAND_MAX)<0.95?-1:0);
	layers->exclusion[i]=(((double) rand()/RAND_MAX)<0.95?0:-1);
	layers->NdiameterReductions[i]=80.+40.*((double) rand()/RAND_MAX);
	layers->diameterReduction[i]=(0.3+0.6*((double) rand()/RAND_MAX));
    layers->object_type[i]=-1+((int)(5*((double) rand()/RAND_MAX)));
	layers->ellipse_ratio[i]=(0.3+0.7*((double) rand()/RAND_MAX));
	layers->r[i]=((double) rand()/RAND_MAX)<0.5?0:1.+5.*((double) rand()/RAND_MAX);
    for(j=0;j<3;j++) layers->color[i][j]=-1;
	layers->Ntrials[i]=80+50*((double) rand()/RAND_MAX);
	layers->p[i][0][0]=image.width_float*((double) rand()/RAND_MAX);
	layers->p[i][0][1]=image.height_float*((double) rand()/RAND_MAX);
	layers->p[i][0][2]=((int) (2.*((double) rand()/RAND_MAX)));
	if( ((double) rand()/RAND_MAX) < 0.6 ){
	  if( ((double) rand()/RAND_MAX) < 0.3){
	    layers->p[i][1][0]=image.width_float*((double) rand()/RAND_MAX);
	    layers->p[i][1][1]=image.height_float*((double) rand()/RAND_MAX);
	    layers->p[i][1][2]=((int) (2.*((double) rand()/RAND_MAX)));
      }
	  else{
		if( ((double) rand()/RAND_MAX) < 0.5){
		  layers->p[i][0][0]=0; layers->p[i][0][1]=0; layers->p[i][0][2]=0;
          layers->p[i][1][0]=0.; layers->p[i][1][1]=0; layers->p[i][1][2]=0;
	    }
		else{
		  layers->p[i][0][0]=0; layers->p[i][0][1]=1; layers->p[i][0][2]=0;
          layers->p[i][1][0]=1.; layers->p[i][1][1]=0; layers->p[i][1][2]=0;
	    }
	  }
    }
	else{
		layers->p[i][1][0]=layers->p[i][0][0];
		layers->p[i][1][1]=layers->p[i][0][1];
		layers->p[i][1][2]=layers->p[i][0][2];
    }
  }
}

/* FUNCTION TO REMOVE 1 LAYER */
void ami_remove_layer(ami_layers_information *layers)
{
  int Nl;
  layers->Nlayers--;
  Nl=layers->Nlayers;

  layers->min_diameter_ratio=(float *) realloc(layers->min_diameter_ratio, sizeof(float)*layers->Nlayers);
  layers->max_diameter_ratio=(float *) realloc(layers->max_diameter_ratio, sizeof(float)*layers->Nlayers);
  layers->r=(float *) realloc(layers->r,sizeof(float)*layers->Nlayers);
  layers->ellipse_ratio=(float *) realloc(layers->ellipse_ratio,sizeof(float)*layers->Nlayers);
  layers->diameterReduction=(float *) realloc(layers->diameterReduction,sizeof(float)*layers->Nlayers);
  layers->color=(vertex_3 *) realloc(layers->color, sizeof(vertex_3)*layers->Nlayers);
  layers->p=(vertex_3_f *) realloc(layers->p, sizeof(vertex_3_f)*layers->Nlayers);
  layers->object_type=(int *) realloc(layers->object_type, sizeof(int)*layers->Nlayers);
  layers->inclusion=(int *) realloc(layers->inclusion, sizeof(int)*layers->Nlayers);
  layers->exclusion=(int *) realloc(layers->exclusion, sizeof(int)*layers->Nlayers);
  layers->Nobj=(int *) realloc(layers->Nobj, sizeof(int)*layers->Nlayers);
  layers->Ntrials=(int *) realloc(layers->Ntrials, sizeof(int)*layers->Nlayers);
  layers->NdiameterReductions=(int *) realloc(layers->NdiameterReductions,sizeof(int)*layers->Nlayers);
}


/* FUNCTION TO DEFINE RANDOMLY RENDER IMAGE PROPERTIES */
int ami_render_definition(
ami_render  *render, // RENDER PARAMETER STRUCTURE
char name[200]) // PALETTE IMAGE FILE NAME
{
  int width,height;
  time_t timer;
  strcpy(render->palette,name);
  if(ami_read_png(name,&(render->red),&(render->green),
                  &(render->blue),&width,&height)!=0){
    printf("We cannot read Palette Color Image\n");
    exit(-1);
  }
  render->Nc=width*height;

  /* WE FIT RENDER INFORMATION */
  render->contrast=50.; /* minimum contrast between color of father and son objects */
  render->keepFatherInclusion= ((double) rand()/RAND_MAX)<0.5?0:-1; /*
    =0 means that son object are not drawn outside the father objet =-1 the opposite */
  if( ((double) rand()/RAND_MAX)<0.8 ){
    render->min_transparency=0.1+0.6*((double) rand()/RAND_MAX);
    render->max_transparency=
    render->min_transparency+((double) rand()/RAND_MAX)*(1.-render->min_transparency); /*
          maximum of transparency value for the objects */
  }
  else{
    render->min_transparency=0.; /* minimum of transparency value for the objects */
    render->max_transparency=0.;
  }

 // LIGHT SPOT EFFECT DEFINITION (IF light_spot=0) NO LIGTH SPOT EFFECT IS INTRODUCED
  if( ((double) rand()/RAND_MAX)<0.5 ){
     render->light_spot=0.5+1.5*((double) rand()/RAND_MAX);
  }
  else if( ((double) rand()/RAND_MAX)<0.3 ){
     render->light_spot=0.;
  }
  else{ //tesselation
    render->light_spot=-1.;
  }

  // BACKGROUND COLOR
  if( ((double) rand()/RAND_MAX)<0.9 ){
     render->background_color[0]=-1;
     render->background_color[1]=-1;
     render->background_color[2]=-1;
  }
  else{
     render->background_color[0]=255;
     render->background_color[1]=255;
     render->background_color[2]=255;
  }
  render->seed=(unsigned int) time(&timer);

  return(0);
}

/* FUNCTION TO READ BASIC IMAGE INFORMATION */
int ami_read_basic_image_information(
char file_name[200],
unsigned int *graine,
ami_image_information *image,
ami_layers_information *layers,
ami_render *render)
{
  FILE *f;
  int k;
  char *palette,strp[200],c='o';
  // WE INITIALIZE RANDOM CHOICE
  time_t timer;
  //*graine = (unsigned int) time(&timer);
  //srand(*graine);

  if(f=fopen(file_name, "r"),!f){
    printf("I cannot open the input file \n");
    return(-1);
  }
  // WE DETECT FIRST TWO "=" SYMBOL IN THE TEXT WHICH CORRESPOND TO COMMENTS
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';

  // WE CREATE DEFAULT IMAGE VALUES
  ami_image_definition(image);

  // WE DETECT SEED PARAMETER
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp);

  if(strp[0]=='*') *graine = (unsigned int) time(&timer);
  else *graine = (unsigned int) atoi(strp);
  srand(*graine);

  printf("seed = %i\n",*graine);

  //while(c!='=' && c!=EOF) {c=getc(f);} fscanf(f,"%lf",&pasof);
  //printf("pasof = %lf\n",pasof);
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') image->width_float=atof(strp);
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') image->height_float=atof(strp);
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') image->max_number_objects=atoi(strp);
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') image->max_diameter=atof(strp);
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') image->Nlayers=atoi(strp);

  //image->color[0]=255; image->color[1]=255; image->color[2]=255;

  while(c!='=' && c!=EOF) {c=getc(f);}  c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') layers->Nlayers=atoi(strp);

  // WE CREATE DEFAULT LAYER VALUES
  ami_layer_definition(layers,layers->Nlayers,*image);

 /* ami_malloc1d(layers->min_diameter_ratio, float,layers->Nlayers);
  ami_malloc1d(layers->max_diameter_ratio, float,layers->Nlayers);
  ami_malloc1d(layers->object_type, int ,layers->Nlayers);
  ami_malloc1d(layers->color, vertex_3,layers->Nlayers);
  ami_malloc1d(layers->inclusion, int ,layers->Nlayers);
  ami_malloc1d(layers->exclusion, int ,layers->Nlayers);
  ami_malloc1d(layers->Nobj, int ,layers->Nlayers);
  ami_malloc1d(layers->Ntrials, int ,layers->Nlayers);
  ami_malloc1d(layers->p, vertex_3_f,layers->Nlayers);
  ami_malloc1d(layers->r,float,layers->Nlayers);
  ami_malloc1d(layers->ellipse_ratio,float,layers->Nlayers);
  ami_malloc1d(layers->NdiameterReductions,int,layers->Nlayers);
  ami_malloc1d(layers->diameterReduction,float,layers->Nlayers);*/


  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++){
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->min_diameter_ratio[k]=atof(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++){
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->max_diameter_ratio[k]=atof(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++) {
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->object_type[k]=atoi(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++) {
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->inclusion[k]=atoi(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++){
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->exclusion[k]=atoi(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++){
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->Nobj[k]=atoi(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++){
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->Ntrials[k]=atoi(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++){
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->p[k][0][0]=atof(strp);
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->p[k][0][1]=atof(strp);
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->p[k][0][2]=atof(strp);
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->p[k][1][0]=atof(strp);
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->p[k][1][1]=atof(strp);
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->p[k][1][2]=atof(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++){
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->r[k]=atof(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++){
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->ellipse_ratio[k]=atof(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++){
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->NdiameterReductions[k]=atoi(strp);
  }
  while(c!='=' && c!=EOF) {c=getc(f);} c='o';
  for(k=0;k<layers->Nlayers;k++){
    fscanf(f,"%s",strp); if(strp[0]!='*') layers->diameterReduction[k]=atof(strp);
  }

  // DEFAULT VALUE FOR render
  palette=&(render->palette[0]);
  while(c!='=' && c!=EOF) {c=getc(f);}  c='o';
  fscanf(f,"%s\n",palette);
  ami_render_definition(render,palette);

  while(c!='=' && c!=EOF) {c=getc(f);}  c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') render->contrast=atof(strp);
  while(c!='=' && c!=EOF) {c=getc(f);}  c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') render->keepFatherInclusion=atoi(strp);
  while(c!='=' && c!=EOF) {c=getc(f);}  c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') render->min_transparency=atof(strp);
  while(c!='=' && c!=EOF) {c=getc(f);}  c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') render->max_transparency=atof(strp);
  while(c!='=' && c!=EOF) {c=getc(f);}  c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') render->light_spot=atof(strp);
  while(c!='=' && c!=EOF) {c=getc(f);}  c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') render->background_color[0]=atoi(strp);
  fscanf(f,"%s",strp); if(strp[0]!='*') render->background_color[1]=atoi(strp);
  fscanf(f,"%s",strp); if(strp[0]!='*') render->background_color[2]=atoi(strp);
  while(c!='=' && c!=EOF) {c=getc(f);}  c='o';
  fscanf(f,"%s",strp); if(strp[0]!='*') render->seed=atoi(strp);

  return(0);
}

/* FUNCTION TO WRITE BASIC IMAGE INFORMATION */
int ami_write_basic_image_information(
char file_name[200],
char image_name[200],
unsigned int
graine,
char **argv,
int argc,
ami_image_information image,
ami_layers_information layers,
ami_render render)
{
  FILE *f;
  int k;
  if(f=fopen(file_name, "w"),!f){
  printf("I cannot open the output file \n");
  return(-1);}

  fprintf(f,"This file contains all configuration parameters used to create the output image\n");
  fprintf(f,"Parameters appear in the file after symbol \"=\".\n");
  fprintf(f,"To reproduce similar images you can change any of the parameters, reload the parameter file\n");
  fprintf(f,"and run the program again. If you want any of the parameter be chosen randomly replace the  \n");
  fprintf(f,"parameter value by the symbol * \n");
  fprintf(f,"You can modify any part of the text. Please do not add extra \"=\" symbols because\n");
  fprintf(f,"the parameter reading procedure will fail\n");

  //fprintf(f,"%s\n\n",image_name);
  printf("%s\n\n",image_name);
  //fprintf(f,"Command line call used to create the image\n");
  //for(k=0;k<argc;k++) fprintf(f,"%s ",argv[k]); fprintf(f,"\n");
  for(k=0;k<argc;k++) printf("%s ",argv[k]); fprintf(f,"\n");

  fprintf(f,"seed used to generate random choice = %u\n", graine);

  //fprintf(f,"To reproduce the same picture execute : \n");
  //for(k=0;k<3;k++) fprintf(f,"%s ",argv[k]);
  //fprintf(f,"%s ",file_name);
  //for(k=4;k<argc;k++) fprintf(f,"%s ",argv[k]);

  fprintf(f,"\nami_image_information parameters\n");
  fprintf(f,"%s %1.9lf  (Normalized image width)\n","image.width_float = ",image.width_float);
  fprintf(f,"%s %1.9lf  (Normalized image height)\n","image.height_float =", image.height_float);
  fprintf(f,"%s %d  (Maximum number of objects)\n","image.max_number_objects = ",
          image.max_number_objects);
  fprintf(f,"%s %1.9lf  (Reference object diameter size),\n",
          "image.max_diameter = ",image.max_diameter );
  fprintf(f,"all other objects diameter are proportional to this one\n");
  fprintf(f,"%s %d  (Number of layers (should be equal to layers.Nlayers)) \n\n","image.Nlayers = ",
          image.Nlayers);

  fprintf(f,"ami_layers_information parameters\n");
  fprintf(f,"%s %d (Number of layers (should be equal to image.Nlayers)) \n","layers.Nlayers = ",
          layers.Nlayers);

  fprintf(f,"\nMinimum object diameter in the layers\n");
  fprintf(f,"%s ","layers.min_diameter_ratio[k] = ");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%1.9lf ",layers.min_diameter_ratio[k]); fprintf(f,"\n");

  fprintf(f,"\nMaximum object diameter in the layers\n");
  fprintf(f,"%s ","layers.max_diameter_ratio[k] = ");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%1.9lf ",layers.max_diameter_ratio[k]);  fprintf(f,"\n");

  fprintf(f,"\nObject type. Values could be : -1 (random choice), 0 (ellipse), 1 (triangle),\n");
  fprintf(f,"2 (rectangle), 3 (N regular polygon),\n");
  fprintf(f,"4 (N irregular polygon),\n");
  fprintf(f,"%s ","layers.object_type[k] = ");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%d ",layers.object_type[k]);  fprintf(f,"\n");

  fprintf(f,"\nInclusion parameter. Values could be : 0 (objects have to be included in \n");
  fprintf(f,"the father object, -1 (otherwise) \n");
  fprintf(f,"%s ","layers.inclusion[k] = ");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%d ",layers.inclusion[k]);  fprintf(f,"\n");

  fprintf(f,"\nExclusion parameter. Values could be : 0 (objects of the layer cannot \n");
  fprintf(f,"intersect), -1 (otherwise) \n");
  fprintf(f,"%s ","layers.exclusion[k] = ");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%d ",layers.exclusion[k]);  fprintf(f,"\n");

  fprintf(f,"\nMaximum number of objects for each father object in the previous layer \n");
  fprintf(f,"%s ","layers.Nobj[k] = ");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%d ",layers.Nobj[k]);  fprintf(f,"\n");

  fprintf(f,"\nMaximum number of trials to include a new object in the layer \n");
  fprintf(f,"%s ","layers.Ntrials[k] = ");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%d ",layers.Ntrials[k]);  fprintf(f,"\n");

  fprintf(f,"\nVanishing points and object orientations. By object orientation we means the orientation of its bounding ellipse\n");
  fprintf(f,"\nWe use auxiliary points (p[0],p[1],p[2]) in projective coordinates  \n");
  fprintf(f,"If (p[0],p[1],p[2]) is equal to (0,0,0). The object orientation is chosen randomly \n");
  fprintf(f,"If p[2] is equal to 0. Objects are oriented in the direction (p[0],p[1])\n");
  fprintf(f,"If p[2] is not equal to 0. Objects point toward the vanishing point (p[0]/p[2],p[1]/p[2])\n");
  fprintf(f,"In this case the size of the object is reduced according to the distance to the vanishing point.\n");
  fprintf(f,"Each layer is associated two vanishing points p. For each object one of both options is chosen randomly. \n");
  fprintf(f,"%s","layers.p[k] = \n");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%lf %lf %lf %lf %lf %lf\n",
  layers.p[k][0][0],layers.p[k][0][1],layers.p[k][0][2],
  layers.p[k][1][0],layers.p[k][1][1],layers.p[k][1][2]);

  fprintf(f,"\nReduction factor of object size when approaching the vanishing point\n");
  fprintf(f,"%s ","layers.r[k] = ");
  for(k=0;k<layers.Nlayers;k++)  fprintf(f,"%1.9lf ",layers.r[k]); fprintf(f,"\n");

  fprintf(f,"\nAspect ratio of the ellipse including the object. Values could be : 0 \n");
  fprintf(f,"(the aspect ratio is chosen randomly) or a number bigger than 0.\n");
  fprintf(f,"(If the value is 1 we keep the original aspect ratio of the object)\n");
  fprintf(f,"%s ","layers.ellipse_ratio[k] = ");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%1.9lf ",layers.ellipse_ratio[k]); fprintf(f,"\n");

  fprintf(f,"\nNumber of diameter reductions allowed to apply if an object cannot be included after the maximum of trials\n");
  fprintf(f,"%s ","layers.NdiameterReductions[k] = ");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%d ",layers.NdiameterReductions[k]); fprintf(f,"\n");

  fprintf(f,"\nReduction factor of object diameter applies to try to include new objects in the image  \n");
  fprintf(f,"%s ","layers.diameterReduction[k] = ");
  for(k=0;k<layers.Nlayers;k++) fprintf(f,"%1.9lf ",layers.diameterReduction[k]);  fprintf(f,"\n\n");

  fprintf(f,"Render parameter definition.\n");
  fprintf(f,"%s %s (Colour palette image name)\n","render.palette = ",render.palette);
  fprintf(f,"%s %1.9lf (Minimum contrast between father and son object colour)\n",
          "render.contrast = ",render.contrast);
  fprintf(f,"%s %d (Values are 0 (objects are not drawn outside father object)), -1 (otherwise) ) \n",
          "render.keepFatherInclusion = ",render.keepFatherInclusion);
  fprintf(f,"%s %1.9lf (Minimum transparency value) \n","render.min_transparency = ",
          render.min_transparency);
  fprintf(f,"%s %1.9lf (Maximum transparency value) \n","render.max_transparency = ",
          render.max_transparency);
  fprintf(f,"%s %1.9lf (Light spot effect, value 0. means no ligth spot effect) \n(if value <0. we color the image tesselation) \n","render.ligth_spot = ",
          render.light_spot);
  fprintf(f,"%s %d %d %d (Background color, if some of the valures are negative the color is taken from the palette) \n",
       "render.background_color = ",render.background_color[0],render.background_color[1],render.background_color[2]);
  fprintf(f,"seed used to generate color random choice = %u\n", render.seed);

  return(0);
}



/* FUNCTION TO DEFINE THE IMAGE NAME BASED ON THE DATE */
void ami_date_name(char name[200])
{
  time_t timer;
  int m,j;
  char name2[200];
  time(&timer);
  m=((localtime(&timer))->tm_year+1900)*10000;
  m+=((localtime(&timer))->tm_mon+1)*100;
  m+=(localtime(&timer))->tm_mday;
  sprintf(name,"%d",m);

  m=1000000;
  m+=((localtime(&timer))->tm_hour)*10000;
  m+=((localtime(&timer))->tm_min)*100;
  m+=(localtime(&timer))->tm_sec;

  j=100000;
  while(j>=1){
	sprintf(name2,"%d",((int) ((int)(m/j))%10));
	j/=10;
    strcat(name,name2);
  }
  printf("name=%s\n",name);
}

/* FUNCTION TO FREE OBJECT MEMORY */
void ami_free_object(ami_object *obj)
{
  (*obj).Nvert=0;
  if((*obj).x!=NULL) free((*obj).x);
  if((*obj).y!=NULL) free((*obj).y);
}

/* FUNCTION TO FREE THE MEMORY OF OBJECT STRUCTURE */
void ami_free_objects(ami_object **obj, int Nobj)
{
  int i;
  for(i=0;i<Nobj;i++){
	  if((*obj)[i].x!=NULL)  free((*obj)[i].x);
	  if((*obj)[i].y!=NULL)  free((*obj)[i].y);
  }
  if((*obj)!=NULL) free(*obj);
}
/* FUNCTION TO FREE THE MEMORY OF LAYER SCTRUCTURE */
void ami_free_layer(ami_layers_information *layers)
{
  int m;
  if(layers->min_diameter_ratio!=NULL) free(layers->min_diameter_ratio);
  if(layers->max_diameter_ratio!=NULL) free(layers->max_diameter_ratio);
  if(layers->object_type!=NULL) free(layers->object_type);
  if(layers->color!=NULL) free(layers->color);
  if(layers->inclusion!=NULL) free(layers->inclusion);
  if(layers->exclusion!=NULL) free(layers->exclusion);
  if(layers->Nobj!=NULL) free(layers->Nobj);
  if(layers->Ntrials!=NULL) free(layers->Ntrials);
  if(layers->p!=NULL) free(layers->p);
  if(layers->r!=NULL) free(layers->r);
  if(layers->ellipse_ratio!=NULL) free(layers->ellipse_ratio);
  if(layers->NdiameterReductions!=NULL) free(layers->NdiameterReductions);
  if(layers->diameterReduction!=NULL) free(layers->diameterReduction);

  if(layers->width!=NULL) free(layers->width);
  if(layers->height!=NULL) free(layers->height);
  if(layers->red!=NULL){
    for(m=0;m<layers->Nlayers;m++){
      free(layers->red[m]);
    }
    free(layers->red);
  }
  if(layers->green!=NULL){
    for(m=0;m<layers->Nlayers;m++){
      free(layers->green[m]);
    }
    free(layers->green);
  }
  if(layers->blue!=NULL){
    for(m=0;m<layers->Nlayers;m++){
      free(layers->blue[m]);
    }
    free(layers->blue);
  }


}


/* FUNCTION TO FREE THE MEMORY OF RENDER SCTRUCTURE */
void ami_free_render(ami_render *render)
{
  if(render->red!=NULL) free(render->red);
  if(render->green!=NULL) free(render->green);
  if(render->blue!=NULL) free(render->blue);
}

/* FUNCTION TO RENDER AN IMAGE */
int ami_render_image(int Nobj,ami_object *obj,ami_render render,ami_image_information image,
            unsigned char *red,unsigned char *green,unsigned char *blue,int width,int height)
{
  int i,j,color_trial,k,m;
  float layer_x=width/image.width_float;
  float layer_y=height/image.height_float;
  float x,y,min_x,min_y,max_x,max_y,x2,y2;
  int size=width*height;
  int Max_Color_Trials=1000; // NUMBER OF PALETTE COLOR RANDOM CHOICE TO SELECT A COLOR
  float light_spot_n,dist,H,S,V,V2;
  unsigned char R,G,B,WB,BC=1;
  unsigned int s1;
  int color_index_size=256*6;
  int color_index[256*6];
  //WE INITIALIZE COLOR RANDOM CHOICE
  srand(render.seed);

  //WE BUILD COLOR INDEX SEQUENCE FOR CONNECTED COMPONENT COLOR if(render.light_spot<0.)
   if(render.light_spot<0.){
     for(k=0;k<color_index_size;k++){
        color_index[k]=((double) rand()/RAND_MAX)*render.Nc;
        color_index[k]=color_index[k]<render.Nc?color_index[k]:render.Nc-1;
     }
   }
  /* WE FIT COLOR BACKGROUND TO THE FIRST OBJECT */
  obj[0].color[0]=render.background_color[0];
  obj[0].color[1]=render.background_color[1];
  obj[0].color[2]=render.background_color[2];

  if(render.background_color[0]>=0 && render.background_color[1]>=0 && render.background_color[2]>=0){
    BC=0;
    for(m=0;m<size;m++){
      red[m]=render.background_color[0];
      green[m]=render.background_color[1];
      blue[m]=render.background_color[2];
    }
  }

  // WE DETECT IF WE DEAL WITH BLACK AND WHITE PALETTE
  WB=0;
  for(i=0;i<render.Nc;i++){
    if(render.red[i]>20 &&  render.red[i]<235){
      WB=1;
      break;
    }
    if(render.green[i]>20 &&  render.green[i]<235){
      WB=1;
      break;
    }
    if(render.blue[i]>20 &&  render.blue[i]<235){
      WB=1;
      break;
    }
  }

  /* WE FILL TRANSPARENCY VALUE */
  obj[0].transparency=0;
  for(i=1;i<Nobj;i++){
   if(obj[i].father_object==0 || WB==0){
     obj[i].transparency=0;
   }
   else{
     obj[i].transparency=render.min_transparency+
     (render.max_transparency-render.min_transparency)*((double) rand()/RAND_MAX);
   }
  }

  /* WE FILL COLOR INFORMATION */
  if(render.Nc==0){
    printf("THERE IS NOT PALETTE COLOR TO RENDER\n");
    return(-1);
  }
  for(i=0;i<Nobj;i++){
    color_trial=0;
    while(color_trial++ < Max_Color_Trials){
      j=((double) rand()/RAND_MAX)*render.Nc;
      j=j<render.Nc?j:render.Nc-1;
      if( i!=0 && 0<=obj[i].father_object && obj[i].father_object<Nobj){
        if( ami_abs((double) render.red[j] - (double) obj[obj[i].father_object].color[0])>=render.contrast ||
          ami_abs((double) render.green[j]- (double) obj[obj[i].father_object].color[1])>=render.contrast ||
          ami_abs((double) render.blue[j]- (double) obj[obj[i].father_object].color[2])>=render.contrast){
            obj[i].color[0]=render.red[j];
            obj[i].color[1]=render.green[j];
            obj[i].color[2]=render.blue[j];
            color_trial=Max_Color_Trials+3;
        }
      }
      else{
        color_trial=Max_Color_Trials+3;
        if(i==0 && BC==0){
          obj[i].color[0]=render.background_color[0];
          obj[i].color[1]=render.background_color[1];
          obj[i].color[2]=render.background_color[2];
        }
        else{
          obj[i].color[0]=render.red[j];
          obj[i].color[1]=render.green[j];
          obj[i].color[2]=render.blue[j];
        }
      }
    }
    if(color_trial < (Max_Color_Trials+3)){
      printf("ami_render_image(): Problem with palette color. Not enough different colors\n");
      return(-1);
    }
  }

  /* RENDERING THE IMAGE */
  for(k=0;k<Nobj;k++){
    if(k==0 && render.background_color[0]==254 &&
        render.background_color[1]==254 && render.background_color[1]==254) continue;
     printf("\robject %d of %d", (k+1), Nobj); fflush(stdout);
	/* WE COMPUTE THE OBJECT BOUNDS */
    ami_max_vector(obj[k].x,i,max_x,obj[k].Nvert);
    ami_max_vector(obj[k].y,i,max_y,obj[k].Nvert);
    ami_min_vector(obj[k].x,i,min_x,obj[k].Nvert);
    ami_min_vector(obj[k].y,i,min_y,obj[k].Nvert);
    // WE CHECK LIGHT SPOT EFFECT
    if(render.light_spot>0.){
      // WE CONVERT OBJECT RGB COLOR TO HSV
      R=obj[k].color[0]; G=obj[k].color[1]; B=obj[k].color[2];
      ami_rgb_to_hsv(R,G,B,&H,&S,&V);
      //AAA
      // WE REDUCE LIGHT SPOT EFFECT ACCORDING TO OBJECT DIAMETER
      light_spot_n=render.light_spot/obj[k].D;
      x2=min_x+((double) rand()/RAND_MAX)*(max_x-min_x);
      y2=min_y+((double) rand()/RAND_MAX)*(max_y-min_y);
      for(i=min_x*layer_x; i<(max_x*layer_x+1);i++){
	    for(j=min_y*layer_y; j<(max_y*layer_y+1);j++){
		  if(0<=i && i<width && j<height && 0<=j){
		    x=i/layer_x; y=j/layer_y;
            //AAA
		    if( ami_test_point_object(x,y,obj[k])==0 ){
			  if( render.keepFatherInclusion==0 && 0<=obj[k].father_object && obj[k].father_object<Nobj){
			    if(ami_test_point_object(x,y,obj[obj[k].father_object])==0){
                  dist=ami_dist2(x,y,x2,y2)*light_spot_n;
                  V2=V*(1./(1.+light_spot_n*dist));
                  ami_hsv_to_rgb(H,S,V2,&R,&G,&B);
			      red[j*width+i]=red[j*width+i]*obj[k].transparency+R*(1.-obj[k].transparency);
                  green[j*width+i]=green[j*width+i]*obj[k].transparency+G*(1.-obj[k].transparency);
                  blue[j*width+i]=blue[j*width+i]*obj[k].transparency+B*(1.-obj[k].transparency);
			    }
			  }
			  else{
                dist=ami_dist2(x,y,x2,y2)*light_spot_n;
                V2=V*(1./(1.+light_spot_n*dist));
                ami_hsv_to_rgb(H,S,V2,&R,&G,&B);
			    red[j*width+i]=red[j*width+i]*obj[k].transparency+R*(1.-obj[k].transparency);
                green[j*width+i]=green[j*width+i]*obj[k].transparency+G*(1.-obj[k].transparency);
                blue[j*width+i]=blue[j*width+i]*obj[k].transparency+B*(1.-obj[k].transparency);
		      }
            }
          }
        }
      }
    }

    else if(render.light_spot==0.){
      for(i=min_x*layer_x; i<(max_x*layer_x+1);i++){
	    for(j=min_y*layer_y; j<(max_y*layer_y+1);j++){
		  if(0<=i && i<width && j<height && 0<=j){
		    x=i/layer_x; y=j/layer_y;
		    if( ami_test_point_object(x,y,obj[k])==0 ){
			  if( render.keepFatherInclusion==0 && 0<=obj[k].father_object && obj[k].father_object<Nobj){
			    if(ami_test_point_object(x,y,obj[obj[k].father_object])==0){
			      red[j*width+i]=red[j*width+i]*obj[k].transparency+obj[k].color[0]*(1.-obj[k].transparency);
                  green[j*width+i]=green[j*width+i]*obj[k].transparency+obj[k].color[1]*(1.-obj[k].transparency);
                  blue[j*width+i]=blue[j*width+i]*obj[k].transparency+obj[k].color[2]*(1.-obj[k].transparency);
			    }
			  }
			  else{
			    red[j*width+i]=red[j*width+i]*obj[k].transparency+obj[k].color[0]*(1.-obj[k].transparency);
                green[j*width+i]=green[j*width+i]*obj[k].transparency+obj[k].color[1]*(1.-obj[k].transparency);
                blue[j*width+i]=blue[j*width+i]*obj[k].transparency+obj[k].color[2]*(1.-obj[k].transparency);
		      }
            }
          }
        }
      }
    }
   else{
      for(i=min_x*layer_x; i<(max_x*layer_x+1);i++){
	    for(j=min_y*layer_y; j<(max_y*layer_y+1);j++){
		  if(0<=i && i<width && j<height && 0<=j){
            s1=(unsigned int) red[j*width+i]+green[j*width+i]+blue[j*width+i]+
                                 obj[k].color[0]+obj[k].color[1]+obj[k].color[2];
            R=render.red[color_index[s1]];
            G=render.green[color_index[s1]];
            B=render.blue[color_index[s1]];

		    x=i/layer_x; y=j/layer_y;
		    if( ami_test_point_object(x,y,obj[k])==0 ){
			  if( render.keepFatherInclusion==0 && 0<=obj[k].father_object && obj[k].father_object<Nobj){
			    if(ami_test_point_object(x,y,obj[obj[k].father_object])==0){
			      red[j*width+i]=R;
                  green[j*width+i]=G;
                  blue[j*width+i]=B;
			    }
			  }
			  else{
			    red[j*width+i]=R;
                green[j*width+i]=G;
                blue[j*width+i]=B;
		      }
            }
          }
        }
      }
    }
  }
  return(0);
}



/* FUNCTION TO GENERATE AN OBJECT*/
int object_generator(
ami_object *obj, /* OUTPUT OBJECT */
int type, /* TYPE OF OBJECT
     type=0 : ellipse defined with Np_Ellipse points
	 type=1 : triangle
	 type=2 : rectangle
	 type=3 : N regular point object with 4<=N<= 10
	 type=4 : N irregular distributed point object with 4<=N<= 10 */
float min_diameter, /* MINIMUM DIAMETER OF THE object */
float max_diameter, /* MAXIMUM DIAMETER OF THE OBJECT */
float w_xmin, float w_ymin, float w_xmax, float w_ymax, /* WINDOW COORDINATES TO SEARCH THE OBJECT */
float c[2], /* CENTER OF THE OBJECT (IF NULL WE CHOICE RANDOMLY THE CENTER) */
float p[3], /* POINT IN PROYECTIVE COORDINATES TO INDICATE THE "PUNTO DE FUGA" */
float r,    /* "PUNTO DE FUGA" SIZE REDUCTION FACTOR */
float ellipse_ratio) /* ELLIPSE AXIS RATIO */
{
  int i;
  double alpha,co,si,a,b,d,norm;
  float x0,y0;
  double _PI=3.14159265358979;

  /* IF type=-1 WE CHOOSE THE TYPE RANDOMLY */
  if(type==-1) type=N_object_TYPE*((double) rand()/RAND_MAX);

  /* WE FIT THE TYPE */
  obj->type = type;

  /* WE FIT THE CENTER OF THE OBJECT */
  x0=(obj->t)[0]=w_xmin+rand()*(w_xmax-w_xmin)/RAND_MAX;
  y0=(obj->t)[1]=w_ymin+rand()*(w_ymax-w_ymin)/RAND_MAX;
  if(c!=NULL ){
	if( c[0]>w_xmin ){
      x0=(obj->t)[0]=c[0];
	  y0=(obj->t)[1]=c[1];
    }
  }

  /* WE FIT THE MAIN AXIS OF THE ELLIPSE WHICH CONTAIN THE OBJECT */
  a=obj->D=min_diameter +(max_diameter-min_diameter)*((double) rand()/RAND_MAX);
  b=obj->d=min_diameter +(max_diameter-min_diameter)*((double) rand()/RAND_MAX);
  if(b>a) { d=a; a=obj->D=b; b=obj->d=d; }
  if(ellipse_ratio>0) b=ellipse_ratio*a;

  /* WE FIT THE ORIENTATION OF THE OBJECT FOLLOWING THE "VANISHING POINT" GIVEN BY VECTOR p[]
  IF p[i]==0 THEN THE ORIENTATION IS FITTED RAMDONLY */
  if(p[0]!=0. || p[1]!=0. || p[2]!=0.){
    if(p[2]==0.){
	   norm=sqrt((double) p[0]*p[0]+p[1]*p[1]);
	   co=(obj->o)[0]=p[0]/norm;
	   si=(obj->o)[1]=p[1]/norm;
    }
	else{
	   co=p[0]/p[2]-x0;
	   si=p[1]/p[2]-y0;
	   norm=sqrt((double) co*co+si*si);
	   if(norm<=0){
		  alpha=_PI*((double) rand()/RAND_MAX-0.5);
	      co=(obj->o)[0]=cos(alpha);
	      si=(obj->o)[1]=sin(alpha);
	   }
	   else{
	     co=(obj->o)[0]=co/norm;
	     si=(obj->o)[1]=si/norm;
	   }
	   /* WE REDUCE THE OBJECT SIZE FOLLOWING THE  DISTANCE TO THE VANISHING POINT */
	   a*=(norm+1e-10)/(norm+r);
	   b*=(norm+1e-10)/(norm+r);
    }
  }
  else{
	alpha=_PI*((double) rand()/RAND_MAX-0.5);
	co=(obj->o)[0]=cos(alpha);
	si=(obj->o)[1]=sin(alpha);
  }

  if(type<1){ /* ELLIPSE */
    obj->Nvert=Np_Ellipse;;
	ami_malloc1d(obj->x,float,obj->Nvert);
	ami_malloc1d(obj->y,float,obj->Nvert);
	for(i=0;i<(obj->Nvert-1);i++){
		alpha=i*2.*_PI/(obj->Nvert-1);
		(obj->x)[i]=x0+co*a*cos(alpha)-si*b*sin(alpha);
		(obj->y)[i]=y0+si*a*cos(alpha)+co*b*sin(alpha);
    }
  }

  else if(type==1){ /* TRIANGLE */
    obj->Nvert=4;
	ami_malloc1d(obj->x,float,obj->Nvert);
	ami_malloc1d(obj->y,float,obj->Nvert);

	if(((double) rand()/RAND_MAX)<0.5){
	  alpha=_PI/2.-2.*_PI/3.;
	  (obj->x)[0]=x0+co*a*cos(alpha)-si*b*sin(alpha);
	  (obj->y)[0]=y0+si*a*cos(alpha)+co*b*sin(alpha);

      alpha=_PI/2.-2.*_PI/3.+((double) rand()/RAND_MAX)*4*_PI/3.;
	  (obj->x)[1]=x0+co*a*cos(alpha)-si*b*sin(alpha);
	  (obj->y)[1]=y0+si*a*cos(alpha)+co*b*sin(alpha);

	  alpha=_PI/2.+2.*_PI/3.;
	  (obj->x)[2]=x0+co*a*cos(alpha)-si*b*sin(alpha);
	  (obj->y)[2]=y0+si*a*cos(alpha)+co*b*sin(alpha);
    }
	else{
	   alpha=-_PI/2.+2.*_PI/3.;
	  (obj->x)[0]=x0+co*a*cos(alpha)-si*b*sin(alpha);
	  (obj->y)[0]=y0+si*a*cos(alpha)+co*b*sin(alpha);

      alpha=-_PI/2.+4.*_PI/3.;
	  (obj->x)[1]=x0+co*a*cos(alpha)-si*b*sin(alpha);
	  (obj->y)[1]=y0+si*a*cos(alpha)+co*b*sin(alpha);

	  alpha=-_PI/2.+4.*_PI/3.+((double) rand()/RAND_MAX)*4*_PI/3.;
	  (obj->x)[2]=x0+co*a*cos(alpha)-si*b*sin(alpha);
	  (obj->y)[2]=y0+si*a*cos(alpha)+co*b*sin(alpha);
    }
  }

  else if(type==2){ /* RECTANGLE  */
    obj->Nvert=5;
	ami_malloc1d(obj->x,float,obj->Nvert);
	ami_malloc1d(obj->y,float,obj->Nvert);
	for(i=0;i<(obj->Nvert-1);i++){
		alpha=_PI*(i/2.+0.25);
		(obj->x)[i]=x0+co*a*cos(alpha)-si*b*sin(alpha);
		(obj->y)[i]=y0+si*a*cos(alpha)+co*b*sin(alpha);
    }
  }

  else if(type==3){ /* N REGULAR POINTS OBJECT  WITH 4<=N<=10 */
    obj->Nvert=4+7*((double) rand()/RAND_MAX);
	ami_malloc1d(obj->x,float,obj->Nvert);
	ami_malloc1d(obj->y,float,obj->Nvert);
	for(i=0;i<(obj->Nvert-1);i++){
		alpha=2*_PI*i/(obj->Nvert-1);
		(obj->x)[i]=x0+co*a*cos(alpha)-si*b*sin(alpha);
		(obj->y)[i]=y0+si*a*cos(alpha)+co*b*sin(alpha);
    }
  }

  else if(type == 4){ /* ARBITRARY N POINTS OBJECT  WITH 4<=N<=10 */
    obj->Nvert=4+7*((double) rand()/RAND_MAX);
	ami_malloc1d(obj->x,float,obj->Nvert);
	ami_malloc1d(obj->y,float,obj->Nvert);
	for(i=0;i<(obj->Nvert-1);i++){
		alpha=((double) rand()/RAND_MAX+i)*2*_PI/(obj->Nvert-1);
		(obj->x)[i]=x0+co*a*cos(alpha)-si*b*sin(alpha);
		(obj->y)[i]=y0+si*a*cos(alpha)+co*b*sin(alpha);
    }
  }

  /* WE FIT THE LAST POINT EQUAL TO THE FIRST ONE TO CLOSE THE POLYGON */
  (obj->x)[obj->Nvert-1]=(obj->x)[0];
  (obj->y)[obj->Nvert-1]=(obj->y)[0];

  /* WE INITIALIZE COLOR AND TRANSPARENCY RANDOMLY */
  for(i=0;i<3;i++) (obj->color)[i]=((double) rand()*255./RAND_MAX);
  obj->transparency= ((double) rand()/RAND_MAX);

  return(0);
}


/* FUNCTION TO GENERATE A SET OF objectS WHICH DEFINE AN IMAGE CONFIGURATION */
void ami_image_generator(ami_object **obj_output, /* OBJECT STRUCTURE TO KEEP THE OBJECT INFORMATION */
                         int *Nobj_output, /* NUMBER OF FINAL objectS */
                         ami_image_information image, /* IMAGE INFORMATION */
                         ami_layers_information layers, /* LAYER INFORMATION */
	                     int max_number_objects) /* MAX NUMBER OF objectS */
{
  int i,k,l,cont2,cont1,cont,cont3,cont4,cont_diameterReductions,m,n;
  float xmax,xmin,ymax,ymin,c[2],ratio_diameterReduction;
  ami_object *obj,*obj2;
  double area,size_min,size_max,x2,y2;

  printf("init ami_image_generator()\n");

  /* WE DEFINE THE WHOLE IMAGE AS THE INITIAL OBJECT (A RECTANGLE) */
  obj=(ami_object*)malloc(2*sizeof(ami_object));
  obj[0].Nvert=5; // the number of points is 5 because the first and last object point are equals
  ami_malloc1d(obj[0].x,float,5);
  ami_malloc1d(obj[0].y,float,5);
  obj[0].x[0]=-0.00001; obj[0].y[0]=-0.00001;
  obj[0].x[3]=-0.00001; obj[0].y[3]=image.height_float;
  obj[0].x[2]=image.width_float; obj[0].y[2]=image.height_float;
  obj[0].x[1]=image.width_float; obj[0].y[1]=-0.00001;
  obj[0].x[4]=-0.00001; obj[0].y[4]=-0.00001;
  obj[0].color[0]=255; obj[0].color[1]=255; obj[0].color[2]=255;
  obj[0].transparency=0;
  obj[0].layer=0;
  obj[0].father_object=-1;
  obj[0].type=3;
  obj[0].D=image.width_float;
  obj[0].d=image.height_float;
  obj[0].t[0]=image.width_float/2.; obj[0].t[1]=image.height_float/2.;
  obj[0].o[0]=1.; obj[0].o[1]=0.;

  /* WE START WITH THE OBJECT GENERATION AT EACH LAYER */
  cont1=0; cont2=1; cont=1; ratio_diameterReduction=1;
  for(k=0;k<layers.Nlayers;k++){ //we go trough layers
    printf("Processing layer %d \n",k);
	for(l=cont1;l<cont2;l++){ // we go through objects at each layer
      // WE COMPUTE THE WINDOW COORDINATES WHERE THE OBJECT IS INCLUDED
	  ami_max_vector((obj[l].x),i,xmax,obj[l].Nvert);
      ami_max_vector((obj[l].y),i,ymax,obj[l].Nvert);
      ami_min_vector((obj[l].x),i,xmin,obj[l].Nvert);
      ami_min_vector((obj[l].y),i,ymin,obj[l].Nvert);

	  area=ami_area_object(obj[l]); // we compute object area

	  // WE TRY TO CREATE NEW OBJECTS BY REDUCING OBJECT DIAMETER
	  cont4=0;
	  if(layers.inclusion[k]==0 || layers.exclusion[k]==0){
        ratio_diameterReduction=obj[l].D>0.5?1.:obj[l].D*2.;
	  }
	  //AAA
	  else ratio_diameterReduction=1.;
      for(cont_diameterReductions=0;cont_diameterReductions<layers.NdiameterReductions[k];cont_diameterReductions++){
        //ratio_diameterReduction=pow((double) layers.diameterReduction[k],(double) cont_diameterReductions);
        ratio_diameterReduction*=layers.diameterReduction[k];
	    cont3=0;
	    while( (cont3++)<layers.Ntrials[k] && cont4<(layers.Nobj[k]) /*(cont_diameterReductions+1)*/ ){
		  c[0]=xmin-1;
		  c[1]=ymin-1;
		  // WE IMPOSE THAT THE OBJECT CENTER SHOULD BE INCLUDED IN THE SILHOUETTE
		  if(layers.width!=NULL){
		    while(
              mask_inclusion_test(
                c[0],c[1],
                layers.red[k],
                layers.width[k],layers.height[k],
                image.width_float,image.height_float)!=0)
            {
              c[0]=rand()*image.width_float/RAND_MAX;
              c[1]=rand()*image.height_float/RAND_MAX;
            }
		  }
		  size_min=layers.min_diameter_ratio[k]; size_max=layers.max_diameter_ratio[k];
		  // OBJECT GENERATION
          m=object_generator(
            &(obj[cont]),
            layers.object_type[k],
            ratio_diameterReduction*size_min*image.max_diameter,
            ratio_diameterReduction*size_max*image.max_diameter,
            xmin,ymin,xmax,ymax,
            c,
            layers.p[k][( (int) rand() )%2],
            layers.r[k],layers.ellipse_ratio[k]
          );
          if(m!=0){
            printf("object_generator fails to build an object\n");
            continue;
          }
          //printf("  end object_generator()\n");

          // WE CHECK IF THE OBJECT IS NEARLY INSIDE THE SILHOUETTE
          x2=y2=0.;
          for(n=0;n<obj[cont].Nvert;n++){
            //printf("n=%d x=%lf y=%lf\n",n,obj[cont].x[n],obj[cont].y[n]); system("pause");
            if( layers.width!=NULL &&
                mask_inclusion_test(
                  obj[cont].x[n],
                  obj[cont].y[n],
                  layers.red[k],
                  layers.width[k],layers.height[k],
                  image.width_float,image.height_float)!=0
             )
               break;
             x2+=obj[cont].x[n];
             y2+=obj[cont].y[n];
             if( layers.width!=NULL && n>0 &&
                mask_inclusion_test(
                  0.5*(obj[cont].x[n]+obj[cont].x[n-1]),
                  0.5*(obj[cont].y[n]+obj[cont].y[n-1]),
                  layers.red[k],
                  layers.width[k],layers.height[k],
                  image.width_float,image.height_float)!=0
              )
                break;
              if( layers.width!=NULL && n==0 &&
                mask_inclusion_test(
                  0.5*(obj[cont].x[0]+obj[cont].x[obj[cont].Nvert-1]),
                  0.5*(obj[cont].y[0]+obj[cont].y[obj[cont].Nvert-1]),
                  layers.red[k],
                  layers.width[k],layers.height[k],
                  image.width_float,image.height_float)!=0
              )
                break;
          }
          if(n<obj[cont].Nvert){
              ami_free_object(&(obj[cont]));
              continue;
          }
          else{
            if( layers.width!=NULL &&
                mask_inclusion_test(
                  x2/n,
                  y2/n,
                  layers.red[k],
                  layers.width[k],layers.height[k],
                  image.width_float,image.height_float)!=0
              ){
                ami_free_object(&(obj[cont]));
                continue;
              }
          }

          if(layers.inclusion[k]==0 && ami_test_object_inclusion(obj[cont],obj[l])==-1){
              ami_free_object(&(obj[cont]));
              continue;
          }
		  i=0;
		  if(layers.exclusion[k]==0){
		   for(i=cont2;i<cont;i++) {
			   if(ami_test_object_intersect(obj[cont],obj[i])==0) i=cont+1;
		    }
          }
		  if(i>cont){
		    ami_free_object(&(obj[cont]));
		    continue;
		  }
		  obj[cont].father_object=l;
		  obj[cont].layer=k;
		  for(i=0;i<3;i++) if(layers.color[k][i]!=-1) obj[cont].color[i]=layers.color[k][i];
		  cont4++; cont++;
		  obj2=obj;
		  if(cont>=max_number_objects){
            if(obj[cont-1].Nvert>0) *Nobj_output=cont;
            else *Nobj_output=cont-2;
            *obj_output=obj;
            return;
          }
		  obj=(ami_object*)realloc(obj,(cont+1)*sizeof(ami_object));
		  if(obj==NULL){
		    printf("fallo realloc\n");
			free(obj2->x); free(obj2->y);
			free(obj2);
			*Nobj_output=0;
			exit(-1);
	      }
	      obj[cont].Nvert=0;
	      if(cont%50==0 || cont_diameterReductions>2 ){
	        printf("object[%d]: cont_diameterReductions=%d ratio_diameterReduction=%1.2f trials=%d\n",
                  cont,cont_diameterReductions,ratio_diameterReduction,cont3);
	      }
          cont3=0;
        }
      }
    }
	cont1=cont2; cont2=cont;
  }
  if(obj[cont].Nvert>0) *Nobj_output=cont+1;
  else *Nobj_output=cont;
  *obj_output=obj;

  printf("end ami_image_generator()\n");
}


/* FUNCTION TO TEST OBJECT INCLUSION  (TEST IF obj1 INCLUDED IN obj2  RETURN 0 IF THE OBJECT
   IS INCLUDED, RETURN -1 OTHERWISE */
int ami_test_object_inclusion(ami_object obj1,ami_object obj2)
{
  int i;
  /* If the objects are two ellipses there is a simple test */
  if(obj2.type==1){
	if((ami_dist(obj1.t[0],obj1.t[1],obj2.t[0],obj2.t[1])+obj1.D)<obj2.D)
	  return(0);
  }
  /* we have to check if all the vertices of obj1 are included in obj2 */
  for(i=0;i<(obj1.Nvert-1);i++) {
      if(ami_test_point_object(obj1.x[i],obj1.y[i],obj2)==-1)
      return(-1);
  }
  return(0);
}

/* FUNCTION TO TEST IF 2 OBJECTS INTERSECT (RETURN 0) IF THE objectS INTERSECT AND RETURN -1 OTHERWISE */
int ami_test_object_intersect(ami_object obj1,ami_object obj2)
{
  int i,j;
  /* We test if the two elliptic domains intersect */
  if( ami_dist(obj1.t[0],obj1.t[1],obj2.t[0],obj2.t[1])>(obj1.D+obj2.D) ) return(-1);
  /* We test if each couple of segments of the edges intersect */
  for(i=0;i<(obj1.Nvert-1);i++){
    for(j=0;j<(obj2.Nvert-1);j++){
  	  if(ami_test_segment_intersect(obj1.x[i],obj1.y[i],obj1.x[i+1],obj1.y[i+1],
                                   obj2.x[j],obj2.y[j],obj2.x[j+1],obj2.y[j+1])==0)
        return(0);
    }
  }
  /* in case of inclusion one has to check if the vertices of one object are include in the other */
  for(i=0;i<(obj1.Nvert-1);i++) if(ami_test_point_object(obj1.x[i],obj1.y[i],obj2)==0)
    return(0);
  for(j=0;j<(obj2.Nvert-1);j++) if(ami_test_point_object(obj2.x[j],obj2.y[j],obj1)==0)
    return(0);

  return(-1);
}

/* FUNCTION TO TEST IF 2 SEGMENTS INTERCEPT (RETURN 0) IF THE SEGMENTS INTERCEPT AND RETURN -1 OTHERWISE */
int ami_test_segment_intersect(float x0,float y0,float x1,float y1,float x2,float y2,float x3,float y3)
{
  float a=(x2-x0)*(y1-y0)-(y2-y0)*(x1-x0);
  float b=(x3-x0)*(y1-y0)-(y3-y0)*(x1-x0);
  if(a*b>0) return(-1);
  a=(x0-x2)*(y3-y2)-(y0-y2)*(x3-x2);
  b=(x1-x2)*(y3-y2)-(y1-y2)*(x3-x2);
  if(a*b>0) return(-1);
  return(0);
}

/* FUNCTION TO TEST IF A POINT IS INSIDE A OBJECT (RETURN 0) IF THE POINT IS INSIDE AND RETURN -1 OTHERWISE */
int ami_test_point_object(float x,float y,ami_object obj){
  int i;
  for(i=0;i<(obj.Nvert-1);i++){
	  if( ((x-obj.x[i])*(obj.y[i+1]-obj.y[i])-(y-obj.y[i])*(obj.x[i+1]-obj.x[i]))>=0. ) return(-1);
  }
  return(0);
}


/* FUNCTION TO COMPUTE THE AREA OF A TRIANGLE */
float ami_area_triangle(float x0,float y0,float x1,float y1,float x2, float y2){
  return(0.5*fabs((double) x1*y2-y1*x2-x0*y2+y0*x2+x0*y1-y0*x1));
}

/* FUNCTION TO COMPUTE THE AREA OF A CONVEX OBJECT OR A TRIANGULATED OBJECT */
float ami_area_object(ami_object obj){
  float area=0;
  int i;
  for(i=0;i<(obj.Nvert-3);i++)
    area+=ami_area_triangle(obj.x[0],obj.y[0],obj.x[i+1],obj.y[i+1],obj.x[i+2],obj.y[i+2]);
  return(area);
}


