/*
 * Author    : Luis Alvarez  <lalvarez@dis.ulpgc.es>
              Jean-Michel Morel <morel@cmla.ens-cachan.fr>
 * Copyright : (C) 2009-2013 IPOL Image Processing On Line http://www.ipol.im/
 * License   : CC Creative Commons "Attribution-NonCommercial-ShareAlike"
              see http://creativecommons.org/licenses/by-nc-sa/3.0/es/deed.en
*/


#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include "io_png.h"
#define ami_abs(x) ((x)>0?(x):(-(x)))
#define ami_malloc1d(address,type,size) {int ml; address=(type *) malloc(sizeof(type)*(size)); \
                     if(address==NULL) {printf("fallo malloc\n"); scanf("%d",&ml);}}
#define ami_dist(x0,y0,x1,y1) ( sqrt( (double) (x1-x0)*(x1-x0)+(y1-y0)*(y1-y0)) )
#define ami_dist2(x0,y0,x1,y1) (  (double) (x1-x0)*(x1-x0)+(y1-y0)*(y1-y0) )
#define ami_max_vector(x,pos,max,dim) {int ml; pos=0; max=x[0]; \
          for(ml=1;ml<dim;ml++) if(x[ml]>max) {max=x[ml]; pos=ml;} \
        }
#define ami_min_vector(x,pos,min,dim) {int ml; pos=0; min=x[0]; \
          for(ml=1;ml<dim;ml++) if(x[ml]<min) {min=x[ml]; pos=ml;} \
        }
#define Np_Ellipse 20  // POINTS TO DISCRETIZE THE ELLIPSE
#define N_object_TYPE 5 // NUMBER OF OBJECT TYPES

typedef int vertex_3[3];
typedef float vertex_3_f[2][3];

/* RENDER INFORMATION */
typedef struct{
   int Nc; /* size of color palette to fit the color for the image */
   unsigned char *red; /* color palette information */
   unsigned char *green;
   unsigned char *blue;
   float contrast; /* minimun contrast between color of father and son objects */
   int keepFatherInclusion; /* =0 means that son object are not drawn outside the father objet
                     =-1 the opposite */
   float min_transparency; /* minimun of transparency value for the objects */
   float max_transparency; /* maximun of transparency value for the objects */
   float light_spot; /* Brightness reduction factor in object light spot effect */
   char palette[200]; /* colour palette image name */
   vertex_3 background_color; /* background color =-1 means color is taken from the palette*/
   unsigned int seed; /* seed to initialize random color choice */
}ami_render;


/* OBJECT STRUCTURE */
typedef struct{
   int Nvert; /* Number of points of the object (the last point is repeated) */
   float *x; /* x coordinate of each point of the object */
   float *y; /* y coordinate of each point of the object */
   /* the object is assumed to be sorted in the clockwise sense */
   int color[3]; /* object color */
   float transparency; /* number between 0 and 1 (0 means opacity, and 1 means total transparency */
   int layer; /* layer where the object was defined */
   int father_object; /* index of the father object where the object is included */
   int type; /* type of object (square, ellipse) */
   float D; /* biggest axis of the ellipse */
   float d; /* smaller axis of the ellipse */
   float t[2]; /* coordinates of the center of the ellipse where the object is included */
   float o[2]; /* unit vector which defines the orientation of the ellipse */
}ami_object;

/* STRUCTURE WITH THE INFORMATION ON THE IMAGE */
typedef struct{
  float width_float; /* width in float precision of the image usually equal to 4. */
  float height_float;	/* height in float precision of the image usually equal to 3. */
  int max_number_objects; /* max number of objects allowed */
  float max_diameter; /* global maximun of ellipse semi-axis of the objects */
  //unsigned char color[3]; /* backgroud color */
  //float isoperimetric_ratio; /* min isoperimetric ratio of an object */
  int Nlayers; /* number of layers to generate the image */
}ami_image_information;

typedef struct{
  int Nlayers; /* Number of layers defined (this number must be greater than the
                  one defined in the image information */
  float *min_diameter_ratio; /* min diameter ratio to apply to the original image
                                max_diameter information */
  float *max_diameter_ratio; /* max diameter ratio to apply to the original image
                                max_diameter information */
  int *object_type; /* type of object for the layer =-1 means random choice */
  vertex_3 *color; /* color of the object for the layer =-1 means random color */
  int *inclusion; /* =0 means that objects has to be included in the father object
                     =-1 (no inclusion is assumed) */
  int *exclusion; /* =0 means that objects can not intersect =-1 (objets can intersect) */
  int *Nobj; /* Maximum number of objects in each layer */
  int *Ntrials; /* Maximum number of trials to insert a object */
  vertex_3_f *p;   /* vanishing points in the layer */
  float *r;      /* Size parameter reduction when approaching the vanishing points */
  float *ellipse_ratio; /* ratio of ellipse axis ellipse_ratio[i]<=1
                          (IF <=0 the ratio is fixed randomly */
  int *NdiameterReductions; /* number of sublayers in each layer in order to reduce the size of objects */
  float *diameterReduction; /* ratio of the size reduction between sublayers */
  unsigned char **red; // red image channel of layer silhouette mask
  unsigned char **green; // green image channel of layer silhouette mask
  unsigned char **blue; // blue image channel of layer silhouette mask
  int *width; // image channel width of layer silhouette mask
  int *height; // image channel height of layer silhouette mask
}ami_layers_information;



/**********************************************************************************
                            FUNCTIONS LIST
***********************************************************************************/

/* FUNCTION TO COMPUTE THE AREA OF A TRIANGLE */
float ami_area_triangle(float x0,float y0,float x1,float y1,float x2, float y2);

/* FUNCTION TO COMPUTE THE AREA OF AN OBJECT */
float ami_area_object(ami_object obj);

/* FUNCTION TO TEST IF A POINT IS INSIDE AN OBJECT (RETURN 0) IF THE POINT IS INSIDE
   AND RETURN -1 OTHERWISE */
int ami_test_point_object(float x,float y,ami_object obj);

/* FUNCTION TO TEST IF 2 SEGMENTS INTERSECT (RETURN 0) IF THE SEGMENTS INTERSECT AND
   RETURN -1 OTHERWISE */
int ami_test_segment_intersect(float x0,float y0,float x1,float y1,
                               float x2,float y2,float x3,float y3);

/* FUNCTION TO TEST IF 2 objectS INTERSECT (RETURN 0) IF THE objectS INTERSECT AND
   RETURN -1 OTHERWISE */
int ami_test_object_intersect(ami_object obj1,ami_object obj2);

/* FUNCTION TO TEST OBJECT INCLUSION (RETURN 0 IF obj1 INCLUDED IN obj2 */
int ami_test_object_inclusion(ami_object obj1,ami_object obj2);

/* FUNCTION TO GENERATE AN OBJECT */
int object_generator(ami_object *obj,int type, float min_diameter,float max_diameter,
                     float w_xmin, float w_ymin,float w_xmax, float w_ymax,float c[2],
                     float p[3],float r,float ellipse_ratio);

/* FUNCTION TO RENDER AN IMAGE */
int ami_render_image(int Nobj,ami_object *obj,ami_render render,ami_image_information image,
          unsigned char *red,unsigned char *green,unsigned char *blue,int width,int height);

/* FUNCTION TO FREE THE MEMORY OF RENDER SCTRUCTURE */
void ami_free_render(ami_render *render);

/* FUNCTION TO FREE THE MEMORY OF LAYER SCTRUCTURE */
void ami_free_layer(ami_layers_information *layer);

/* FUNCTION TO FREE THE MEMORY OF objectS SCTRUCTURE */
void ami_free_objects(ami_object **obj, int Nobj);

/* FUNCTION TO DEFINE THE IMAGE NAME BASED ON THE DATE */
void ami_date_name(char name[200]);

/* FUNCTION TO DEFINE RENDER IMAGE PROPERTIES */
int ami_render_definition(ami_render  *render,char name[200]);

/* FUNCTION TO DEFINE IMAGE PARAMETERS */
void ami_image_definition(ami_image_information *image);

/* FUNCTION TO DEFINE LAYER INFORMATION */
void ami_layer_definition(ami_layers_information *layers,int Nlayers,ami_image_information image);

/* FUNCTION TO REMOVE 1 LAYER */
void ami_remove_layer(ami_layers_information *layers);

/* FUNCTION TO ADD 1 LAYER */
void ami_add_layer(ami_layers_information *layers,ami_image_information image);

/* FUNCTION TO FREE object MEMORY */
void ami_free_object(ami_object *obj);

/* FUNCTION TO CREATE AN IMAGE */
void ami_image_generator(ami_object **obj_output,int *Nobj_output,ami_image_information image,
                         ami_layers_information layers,int max_number_objects);
/* FUNCTION TO WRITE BASIC IMAGE INFORMATION */
int ami_write_basic_image_information(char file_name[200],char image_name[200],unsigned int graine,
char **argv, int argc, ami_image_information image, ami_layers_information layers, ami_render render);

/* FUNCTION TO WRITE BASIC IMAGE INFORMATION */
int ami_read_basic_image_information(char file_name[200], unsigned int *graine,
ami_image_information *image,ami_layers_information *layers, ami_render *render);


/* FUNCTION TO CONVERT FROM RGB COLOR SPACE TO HSV */
void ami_rgb_to_hsv(unsigned char red,unsigned char green,unsigned char blue,
                    float *H,float *S,float *V);

/* FUNCTION TO CONVERT FROM HSV COLOR SPACE TO RGB */
void ami_hsv_to_rgb(float h,float s,float v,
                    unsigned char *r,unsigned char *g,unsigned char *b);
